package org.xdv.clx.expr;

import org.xdv.clx.exec.ClxExecutionContext;
import org.xdv.clx.exec.ClxExecutionPoint;
import org.xdv.common.XdvValidationException;

/**
 * Formula definition - a general CLX type.
 * Formula is a desription of constraint in first order logic + XPath (as a substitute of 
 * operations from set theory). 
 * Formula can consist of other formulas. Every formula can be execute in two ways:
 * <ol>
 *  <li>Evaluates formula and gives simple answer: formula is fulfiled or not.
 *  <li>Evaluates formula with posibility to resume evaluation from last point.
 * </ol>   
 *
 * @see <a href="http://www.clixml.org/clix/1.0/#formulae">CLiXML - Constraint formulas</a>
 * @see <a href="http://www.clixml.org/clix/1.0/docs/html/#group_formulaGroup_Link01B42A80">CLiXML - formulaGroup</a>  
 * @author Tomasz Kokoszka
 */
public interface ClxFormula {

    /**
     * Executes formula.
     * @param cntx CLX execution context
     * @return evaluated formula's value
     * @throws XdvValidationException error while executing formula (for instance: error in
     *         XPath expression)
     */
    public boolean executeFormula(ClxExecutionContext cntx)
    throws XdvValidationException;
    
    /**
     * Executes formula starting from given execution point.
     * @param cntx CLX execution context
     * @param frame execution; modified in place. After executed it will store a new execution 
     *        point 
     * @return evaluated formula's value
     * @throws XdvValidationException error while executing formula (for instance: error in
     *         XPath expression)
     */
    public boolean executeFormula(ClxExecutionContext cntx, ClxExecutionPoint frame)
    throws XdvValidationException;

}
