package org.xdv.clx.expr;

import org.xdv.clx.exec.ClxExecutionContext;
import org.xdv.clx.exec.ClxExecutionPoint;
import org.xdv.common.XdvValidationException;

/**
 * OR - CLX element implementation.
 * <br>
 * It is an <i>lazy evaluated</i> version - like Java <code>||</code> operator.
 * 
 * @see <a href="http://www.clixml.org/clix/1.0/#or">CLiXML - clx:or</a>
 * @see org.xdv.clx.base.ClxConstants#TAG_OR
 * @author Tomasz Kokoszka
 */
public class ClxOr extends ClxFormulaBinaryOperator {

    /**
     * Creates instance.
     * @param formula1 first sub-formula
     * @param formula2 secound sub-formula
     */
    public ClxOr(ClxFormula formula1, ClxFormula formula2) {
        super(formula1, formula2);
    }
    
    public boolean executeFormula(ClxExecutionContext cntx) 
    throws XdvValidationException {
        boolean b1 = this.getFormula1().executeFormula(cntx);
        if (! b1) {
            return this.getFormula2().executeFormula(cntx);
        }
        return true;
    }
    
    public boolean executeFormula(ClxExecutionContext cntx, ClxExecutionPoint frame) 
    throws XdvValidationException {
        boolean b;
        final State s = getExecutionState(frame);
        
        if (frame.getNext1().isFinished()) {
            b = s.v1;
        } else {
            b = this.getFormula1().executeFormula(cntx, frame.getNext1());
            s.v1 = b;
        }
                
        if (! b) {
            if (frame.getNext2().isFinished()) {
                b = s.v2;
            } else {
                b = this.getFormula2().executeFormula(cntx, frame.getNext2());
                s.v2 = b;
            }            
            frame.setFinished();
        } else {
            frame.setFinished(frame.getNext1().isFinished());
        }
        
        return b;
    }        

    public String toString() {
        return "(" + getFormula1() + ") => (" + getFormula2() + ")";
    }

}
