package org.xdv.xpath;

/**
 * XPath exception - general problem with XPath (creation, execution, etc).
 * <p>
 * It is a simple extension of {@link java.lang.Exception} class.
 * Additional method {@link #getXpath()} returns xpath expression (as a text) that 
 * caused the problem. Default value for <code>Xpath</code> is <code>null</code>.
 * For detailed description about constructors please look directly to superclass 
 * ({@link Exception}).
 * </p>  
 *
 * @author Tomasz Kokoszka
 */
public class XPathException extends Exception {
    
    /** Exception cause - XPath expression (as a text) related to this exception, can be null */ 
    private String xpath = null;

    /**
     * Constructs a new exception with <code>null</code> as its detail message.
     * @see Exception#Exception()
     */
    public XPathException() {
        super();
    }

    /**
     * Constructs a new exception with the specified detail message and
     * cause.
     * @param  message the detail message; can be null
     * @param  cause the cause; can be null
     * @see Exception#Exception(java.lang.String, java.lang.Throwable)
     */
    public XPathException(String message, Throwable cause) {
        super(message, cause);
    }
    
    /**
     * Constructs a new exception with the specified detail message.  T
     * @param message the detail message; can be null
     * @see Exception#Exception(java.lang.String)
     */
    public XPathException(String message) {
        super(message);                
    }

    /**
     * Constructs a new exception with the specified cause and a detail
     * message of <tt>(cause==null ? null : cause.toString())</tt>.
     * @param  cause the cause; can be null
     * @see Exception#Exception(java.lang.Throwable)
     */
    public XPathException(Throwable cause) {
        super(cause);
    }
    
    /**
     * Constructs a new exception with the specified detail message, xpath expression and cause.  
     * @param message the detail message; can be null
     * @param xpath the XPath expression; can be null
     * @param cause the cause; can be null
     * @see Exception#Exception(java.lang.String, java.lang.Throwable)
     */
    public XPathException(String message, String xpath, Throwable cause) {
        super(message, cause);
        this.xpath = xpath;
    }
    
    /**
     * Constructs a new exception with the specified detail message and xpath expression.
     * @param message the detail message; can be null
     * @param xpath the XPath expression; can be null
     * @see Exception#Exception(java.lang.String)
     */
    public XPathException(String message, String xpath) {
        super(message);                
        this.xpath = xpath;
    }
    
    /**
     * Returns XPath expression (as a text) related to this exception.
     * @return xpath expression as a text; can be null
     */
    public String getXpath() {
        return xpath;
    }

}
